
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_CAMERA_HH_
#define _MOOF_CAMERA_HH_

#include <Moof/Event.hh>
#include <Moof/Frustum.hh>
#include <Moof/Interpolator.hh>
#include <Moof/Math.hh>


namespace Mf {


class Camera
{
	void calculateSecondary();

public:
	Camera() :
		position_(0.0, 0.0, 0.0)
	{
		cml::quaternion_rotation_world_y(rotation_, 0.0);
		calculateSecondary();
	}

	void setPosition(const Vector3& point);
	void setRotation(const Quaternion& rotation);

	void setProjection(const Matrix4& projection);
	void setProjection(Scalar fovy, Scalar aspect, Scalar near, Scalar far);

	void uploadProjectionToGL() const;

	void lookAt(const Vector3& point);

	const Matrix4& getModelviewMatrix() const
	{
		return modelview_;
	}

	const Frustum& getFrustum() const
	{
		return frustum_;
	}

	void handleEvent(const Event& event);
	void update(Scalar t, Scalar dt);

private:
	Vector3 	position_;
	Quaternion	rotation_;
	Matrix4		projection_;

	Matrix4 	modelview_;
	Frustum		frustum_;

	Lerpv3		pInterp_;
};


} // namespace Mf

#endif // _MOOF_CAMERA_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

