
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_RECTANGLE_HH_
#define _MOOF_RECTANGLE_HH_

#include <Moof/Math.hh>


namespace Mf {


/**
 * Axis-aligned Bounding Box
 */

struct Rectangle
{
	Vector2 min;
	Vector2 max;


	Rectangle() {}

	Rectangle(const Vector2& a, const Vector2& b)
	{
		init(a, b);
	}

	Rectangle(Scalar ax, Scalar ay, Scalar bx, Scalar by)
	{
		Vector2 a(ax, ay);
		Vector2 b(bx, by);

		init(a, b);
	}

	inline void init(const Vector2& a, const Vector2& b)
	{
		if (a[0] < b[0])
		{
			min[0] = a[0];
			max[0] = b[0];
		}
		else
		{
			min[0] = b[0];
			max[0] = a[0];
		}
		if (a[1] < b[1])
		{
			min[1] = a[1];
			max[1] = b[1];
		}
		else
		{
			min[1] = b[1];
			max[1] = a[1];
		}
	}

	inline Vector2 getCenter() const
	{
		return Vector2((min[0] + max[0]) / 2.0,
					   (min[1] + max[1]) / 2.0);
	}

	void getCorners(Vector2 corners[4]) const;
};


} // namespace Mf

#endif // _MOOF_RECTANGLE_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

