
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_TEXTURE_HH_
#define _MOOF_TEXTURE_HH_

/**
 * @file Texture.hh
 * Image-loading and OpenGL texture loading.
 */

#include <stdexcept>

#include <boost/shared_ptr.hpp>

#include <Moof/Exception.hh>
#include <Moof/OpenGL.hh>
#include <Moof/Resource.hh>


namespace Mf {


class Texture;
typedef boost::shared_ptr<Texture> TextureP;


class Texture : public Resource
{
	class Impl;
	boost::shared_ptr<Impl> impl_;

public:

	static TextureP alloc(const std::string& name)
	{
		return TextureP(new Texture(name));
	}

	Texture(const std::string& name);

	void bind() const;
	GLuint getObject() const;

	static void resetBind();

	unsigned getWidth() const;
	unsigned getHeight() const;

	void setMinFilter(GLuint filter);
	void setMagFilter(GLuint filter);
	void setWrapS(GLuint wrap);
	void setWrapT(GLuint wrap);

	static std::string getPath(const std::string& name);


	struct Exception : public Mf::Exception
	{
		enum
		{
			BAD_IMAGE_FORMAT	= 1024
		};

		explicit Exception(unsigned error) :
			Mf::Exception(error) {}

		void raise()
		{
			throw *this;
		}

		const char* what() const throw()
		{
			switch (code)
			{
				case BAD_IMAGE_FORMAT:
					return "unknown image format";
			}
			return Mf::Exception::what();
		}
	};
};


} // namespace Mf

#endif // _MOOF_TEXTURE_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

